/*Description:
     Example code to demonstrate DC current measurement (1mA-1000mA).
     Code is written for default MCU for EasyPic7, PIC 18F45K22 @8Mhz

 * Test configuration:
     MCU:             PIC18F45K22
                      http://ww1.microchip.com/downloads/en/DeviceDoc/41412D.pdf
     Dev.Board:       EasyPIC7
                      http://www.mikroe.com/easypic/
     Oscillator:      8.0000 MHz Crystal
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/mikroc/pic/
 */

#include <built_in.h>

// Lcd module connections
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End Lcd module connections

// ADC click module connections
sbit Chip_Select_Direction   at TRISE0_bit;
sbit Chip_Select             at LATE0_bit;
// eof ADC click module connections

unsigned int measurement, lastValue;

char i;
unsigned int adc_rd, sum, average, adc_rd1, adc_rd2, adc_rd3;
int Voltage, Vref, Current;
float Voltage_V;
char txt[4];

unsigned int getADC() {
  // Local variables
  char i, buffer;
  volatile unsigned int read, read1, avrg, sum;

  // Initialize variable
  sum = 0;

  for(i=0; i<10; i++ ) {
    Chip_Select = 0;               // Select MCP3201
    read = SPI1_Read(buffer);      // Get first 8 bits of ADC value
    read1 = SPI1_Read(buffer);     // Get the rest of the ADC value bits
    Chip_Select = 1;               // Deselect MCP3201
    read = ((read & 0x1F) << 8);   // Store the first 8 bits of the ADC value in temporary variable
    read = ((read | read1) >> 1);  // Store the rest of the ADC value bits in temporary variable
    sum = sum + read;              // Sum of the eight ADC readings
  }
  avrg = sum / 10;                  // Average ADC value based on sum of the ADC readings
  return avrg;                     // Returns the average ADC value
}


void main() {

  ANSELA = 4;                     // Configure RA2 pins as analog - Mikro BUS1 4; Mikro BUS2 8;
  ANSELC = 0;                     // Configure PORTC pins as digital
  ANSELD = 0;                     // Configure PORTD pins as digital
  ANSELE = 0;

  TRISA = 0x04;                   // Set RA2 pin as input - Mikro BUS1 RA2 4; Mikro BUS2 RA3 8;
  TRISC = 0x00;                   // Set PORTC as output
  TRISD = 0x00;                   // Set PORTD as output

  UART1_Init(9600);               // Initialize UART module at 9600 bps
  Delay_ms(100);                  // Wait for UART module to stabilize

  ADCON0 =   0b01111101;
  ADCON1 =   0b00001000;
  ADCON2 =   0b10000000;
  VREFCON0 = 0b11100000;

  Voltage = 0;
  Vref = 2048;
  sum = 0;
  measurement = 0;

  Chip_Select = 1;                 // Deselect MCP3204
  Chip_Select_Direction = 0;       // Set chip select pin to be output

   // Initialize SPI1 module at 500kHz, data sampled at the middle of interval
  SPI1_Init_Advanced(_SPI_MASTER_OSC_DIV64, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW, _SPI_LOW_2_HIGH);
  LCD_Init();
  while (1) {

    sum = 0;

for(i=0; i<20; i++ ) {
    adc_rd = ADC_Read(2);             // get ADC value from 2rd channel
    delay_ms(50);
    sum = sum + adc_rd;               // Sum of the eight ADC readings
  }

    average = sum / 20;                   // Average ADC value based on sum of the ADC readings

    Voltage = average * 2;                 // U = (adc_rd*Vref)/1024

    IntToStr(Voltage, txt);

    UART1_Write_Text(txt);
    UART1_Write(32);
    UART1_Write_Text("mV");
    UART1_Write(13);
    UART1_Write(10);

    Current = Voltage - 1016;

    IntToStr(Current, txt);

    UART1_Write_Text(txt);
    UART1_Write(32);
    UART1_Write_Text("mA");
    UART1_Write(13);
    UART1_Write(10);
    
    Lcd_Cmd(_LCD_CURSOR_OFF);
    LCD_Out(2,8,txt);
    LCD_Out(2,14, "mA");

//   SPI
    
    measurement = getADC();        // Get ADC result

    Voltage = measurement/2;

    IntToStr(Voltage, txt);

    UART1_Write_Text(txt);
    UART1_Write(32);
    UART1_Write_Text("mV");
    UART1_Write(13);
    UART1_Write(10);

    Current = Voltage - 1022;

    IntToStr(Current, txt);

    UART1_Write_Text(txt);
    UART1_Write(32);
    UART1_Write_Text("mA");
    UART1_Write(13);
    UART1_Write(10);
    
    Lcd_Cmd(_LCD_CURSOR_OFF);
    LCD_Out(1,8,txt);
    LCD_Out(1,14, "mA");
    LCD_Out(1,1, "Current:");

    delay_ms(1000);
  }
}